==> PyMSNt-0.3.tar.gz: PyMSNt/src/groupchat.py <==
# Copyright 2004 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details

from twisted.internet import reactor
from twisted.xish.domish import Element
import jabw
import config
import debug
import string


class BaseGroupchat:
	""" A class to map a groupchat from a legacy service back to the Jabber user """
	def __init__(self, session, resource, ID=None):
		self.session = session
		self.session.groupchats.append(self)
		self.resource = None
		if(ID):
			self.ID = ID
			self.session.pytrans.reserveID(self.ID)
		else:
			self.ID = self.session.pytrans.makeID()
		
		self.ready = False # Is True only after the user has joined
		self.messageBuffer = []
		self.contacts = []
		
		reactor.callLater(60.0*2, self.checkUserJoined, None)
		
		debug.log("BaseGroupchat: \"%s\" created" % (self.roomJID()))
	
	def removeMe(self):
		""" Cleanly removes the object """
		self.session.groupchats.remove(self)
		self.session.sendPresence(to=self.user(), fro=self.roomJID() + "/me", ptype="unavailable")
		self.session = None
		
		debug.log("BaseGroupchat: \"%s\" destroyed" % (self.roomJID()))
	
	def roomJID(self):
		""" Returns the room JID """
		return self.ID + "@" + config.jid
	
	def user(self):
		""" Returns the full JID of the Jabber user in this groupchat """
		jid = self.session.jabberID
		if(self.resource):
			jid += "/" + self.resource
		return jid
	
	def checkUserJoined(self, ignored=None):
		if(not self.ready):
			debug.log("BaseGroupchat: \"%s\" User hasn't joined after two minutes. Removing them from the room.")
			
			text = []
			text.append("You did not join the groupchat room %s." % (self.roomJID()))
			text.append("The following users were in the groupchat:")
			for contact in self.contacts:
				text.append("\t%s" % (contact))
			text.append("")
			text.append("You have been removed from this room on the legacy service. The following was said before you were disconnected, while you appeared to be in the groupchat to the contacts on the legacy service.")
			text.append("")
			for (source, message) in self.messageBuffer:
				if(source):
					text.append("%s says: %s" % (source, message))
				else:
					text.append(message)
			
			body = string.join(text, "\n")
			
			self.session.sendMessage(to=self.user(), fro=config.jid, body=body)
			
			self.removeMe()
	
	def sendUserInvite(self, fro):
		""" Sends the invitation out to the Jabber user to join this room """
		el = Element((None, "message"))
		el.attributes["from"] = fro
		el.attributes["to"] = self.user()
		body = el.addElement("body")
		text = "You have been invited into a groupchat on the legacy service. You must join this room to switch into groupchat mode %s.\n" % (self.roomJID())
		text += "If you do not join this room you will not be able to participate in the groupchat, but you will still appear to have joined it to contacts on the legacy service."
		body.addContent(text)
		x = el.addElement("x")
		x.attributes["jid"] = self.roomJID()
		x.attributes["xmlns"] = "jabber:x:conference"
		debug.log("BaseGroupchat: \"%s\" sending invitation to \"%s\" to join" % (self.roomJID(), self.user()))
		self.session.pytrans.send(el)
	
	def userJoined(self):
		# Send any buffered messages
		self.session.sendPresence(to=self.user(), fro=self.roomJID() + "/me") # FIXME
		if(not self.ready):
			debug.log("Groupchat: \"%s\" user has joined us!" % (self.roomJID()))
			self.ready = True
			for (source, text) in self.messageBuffer:
				self.messageReceived(source, text)
			self.messageBuffer = None
			for contact in self.contacts:
				self.contactPresenceChanged(contact)
	
	def contactJoined(self, contact):
		if(self.contacts.count(contact) == 0):
			self.contacts.append(contact)
			debug.log("BaseGroupchat: \"%s\" Legacy contact has joined \"%s\"" % (self.roomJID(), contact))
		self.contactPresenceChanged(contact)
		self.messageReceived(None, "%s has joined the conference." % (contact))
	
	def contactLeft(self, contact):
		if(self.contacts.count(contact) > 0):
			self.contacts.remove(contact)
			debug.log("BaseGroupchat: \"%s\" Legacy contact has left \"%s\"" % (self.roomJID(), contact))
		self.contactPresenceChanged(contact, ptype="unavailable")
		self.messageReceived(None, "%s has left the conference." % (contact))
	
	def messageReceived(self, source, message):
		if(not self.ready):
			self.messageBuffer.append((source, message))
		else:
			fro = self.roomJID()
			if(source):
				fro += "/" + source
			debug.log("BaseGroupchat: \"%s\" messageReceived(\"%s\", \"%s\")" % (self.roomJID(), source, message))
			self.session.sendMessage(to=self.user(), fro=fro, body=message, mtype="groupchat")
	
	def contactPresenceChanged(self, contact, ptype=None):
		if(self.session):
			fro = self.roomJID() + "/" + contact
			self.session.sendPresence(to=self.user(), fro=fro, ptype=ptype)
	
	def sendMessage(self, text):
		debug.log("BaseGroupchat: \"%s\" sendMessage(\"%s\")" % (self.roomJID(), text))
		self.messageReceived("me", text)
		self.sendLegacyMessage(text)
	
	def sendLegacyMessage(self, text):
		""" Reimplement this to send the packet to the legacy service """
		pass
	
	def sendContactInvite(self, contact):
		""" Reimplement this to send the packet to the legacy service """
		pass

==> PyMSNt-0.4.tar.gz: PyMSNt/src/groupchat.py <==
# Copyright 2004 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details

from twisted.internet import reactor
from twisted.xish.domish import Element
import jabw
import config
import debug
import string


class BaseGroupchat:
	""" A class to map a groupchat from a legacy service back to the Jabber user """
	def __init__(self, session, resource, ID=None):
		self.session = session
		self.session.groupchats.append(self)
		self.resource = None
		if(ID):
			self.ID = ID
			self.session.pytrans.reserveID(self.ID)
		else:
			self.ID = self.session.pytrans.makeID()
		
		self.ready = False # Is True only after the user has joined
		self.nick = None
		self.messageBuffer = []
		self.contacts = []
		
		reactor.callLater(60.0*2, self.checkUserJoined, None)
		
		debug.log("BaseGroupchat: \"%s\" created" % (self.roomJID()))
	
	def removeMe(self):
		""" Cleanly removes the object """
		self.session.groupchats.remove(self)
		self.session.sendPresence(to=self.user(), fro=self.roomJID() + "/" + self.nick, ptype="unavailable")
		self.session = None
		
		debug.log("BaseGroupchat: \"%s\" destroyed" % (self.roomJID()))
	
	def roomJID(self):
		""" Returns the room JID """
		return self.ID + "@" + config.jid
	
	def user(self):
		""" Returns the full JID of the Jabber user in this groupchat """
		jid = self.session.jabberID
		if(self.resource):
			jid += "/" + self.resource
		return jid
	
	def checkUserJoined(self, ignored=None):
		if(not self.ready):
			debug.log("BaseGroupchat: \"%s\" User hasn't joined after two minutes. Removing them from the room.")
			
			text = []
			text.append("You did not join the groupchat room %s." % (self.roomJID()))
			text.append("The following users were in the groupchat:")
			for contact in self.contacts:
				text.append("\t%s" % (contact))
			text.append("")
			text.append("You have been removed from this room on the legacy service. The following was said before you were disconnected, while you appeared to be in the groupchat to the contacts on the legacy service.")
			text.append("")
			for (source, message) in self.messageBuffer:
				if(source):
					text.append("%s says: %s" % (source, message))
				else:
					text.append(message)
			
			body = string.join(text, "\n")
			
			self.session.sendMessage(to=self.user(), fro=config.jid, body=body)
			
			self.removeMe()
	
	def sendUserInvite(self, fro):
		""" Sends the invitation out to the Jabber user to join this room """
		el = Element((None, "message"))
		el.attributes["from"] = fro
		el.attributes["to"] = self.user()
		body = el.addElement("body")
		text = "You have been invited into a groupchat on the legacy service. You must join this room to switch into groupchat mode %s.\n" % (self.roomJID())
		text += "If you do not join this room you will not be able to participate in the groupchat, but you will still appear to have joined it to contacts on the legacy service."
		body.addContent(text)
		x = el.addElement("x")
		x.attributes["jid"] = self.roomJID()
		x.attributes["xmlns"] = "jabber:x:conference"
		debug.log("BaseGroupchat: \"%s\" sending invitation to \"%s\" to join" % (self.roomJID(), self.user()))
		self.session.pytrans.send(el)
	
	def userJoined(self, nick):
		# Send any buffered messages
		self.nick = nick
		self.session.sendPresence(to=self.user(), fro=self.roomJID() + "/" + self.nick) # FIXME
		if(not self.ready):
			debug.log("Groupchat: \"%s\" user has joined us!" % (self.roomJID()))
			self.ready = True
			for (source, text) in self.messageBuffer:
				self.messageReceived(source, text)
			self.messageBuffer = None
			for contact in self.contacts:
				self.contactPresenceChanged(contact)
	
	def contactJoined(self, contact):
		if(self.contacts.count(contact) == 0):
			self.contacts.append(contact)
			debug.log("BaseGroupchat: \"%s\" Legacy contact has joined \"%s\"" % (self.roomJID(), contact))
		self.contactPresenceChanged(contact)
		self.messageReceived(None, "%s has joined the conference." % (contact))
	
	def contactLeft(self, contact):
		if(self.contacts.count(contact) > 0):
			self.contacts.remove(contact)
			debug.log("BaseGroupchat: \"%s\" Legacy contact has left \"%s\"" % (self.roomJID(), contact))
		self.contactPresenceChanged(contact, ptype="unavailable")
		self.messageReceived(None, "%s has left the conference." % (contact))
	
	def messageReceived(self, source, message):
		if(not self.ready):
			self.messageBuffer.append((source, message))
		else:
			fro = self.roomJID()
			if(source):
				fro += "/" + source
			debug.log("BaseGroupchat: \"%s\" messageReceived(\"%s\", \"%s\")" % (self.roomJID(), source, message))
			self.session.sendMessage(to=self.user(), fro=fro, body=message, mtype="groupchat")
	
	def contactPresenceChanged(self, contact, ptype=None):
		if(self.session):
			fro = self.roomJID() + "/" + contact
			self.session.sendPresence(to=self.user(), fro=fro, ptype=ptype)
	
	def sendMessage(self, text):
		debug.log("BaseGroupchat: \"%s\" sendMessage(\"%s\")" % (self.roomJID(), text))
		self.messageReceived(self.nick, text)
		self.sendLegacyMessage(text)
	
	def sendLegacyMessage(self, text):
		""" Reimplement this to send the packet to the legacy service """
		pass
	
	def sendContactInvite(self, contact):
		""" Reimplement this to send the packet to the legacy service """
		pass
